unit Node;

// base node class

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
	Emitter;

type

	TNode = class
	public
		nodeLeft: TNode;
		nodeRight: TNode;
		constructor Create; overload;
		constructor Create(ANodeLeft: TNode; ANodeRight: TNode); overload;
		destructor Destroy; override;
		procedure Traverse;
		procedure CodeGen; virtual;
		procedure TraverseTokens; virtual;
	end;

procedure CodeGenerate(node: TNode);


// ==========================================================================
implementation

// --------------------------------------------------------------------------
// follow a node if it is defined
procedure CodeGenerate(node: TNode);
begin
	if Assigned(node) then
		node.CodeGen;
end;


{ TNode }

// --------------------------------------------------------------------------
// class constructor
constructor TNode.Create;
begin
	nodeLeft := nil;
	nodeRight := nil;
end;

// --------------------------------------------------------------------------
// class constructor

constructor TNode.Create(ANodeLeft, ANodeRight: TNode);
begin
	nodeLeft := ANodeLeft;
	nodeRight := ANodeRight;
end;

// --------------------------------------------------------------------------
// class destructor
destructor TNode.Destroy;
begin
	if Assigned(nodeLeft) then
		nodeLeft.free;
	if Assigned(nodeRight) then
		nodeRight.free;
end;

// --------------------------------------------------------------------------
// traverse
procedure TNode.Traverse;
begin
{
	memoNodes.Lines.Add( self.ClassName);
	if Assigned(nodeLeft) then
		nodeLeft.Traverse;
	if Assigned(nodeRight) then
		nodeRight.Traverse;
	}
end;

// --------------------------------------------------------------------------
// abstract class output code
procedure TNode.CodeGen;
begin
	//emit.Emit('codegen: ' + self.className());
	CodeGenerate(nodeLeft);
	CodeGenerate(nodeRight);
end;

// --------------------------------------------------------------------------
// read token declarations
procedure TNode.TraverseTokens;
begin
	if Assigned(nodeLeft) then
		nodeLeft.TraverseTokens;
	if Assigned(nodeRight) then
		nodeRight.TraverseTokens;
end;

end.
